require 'sendgrid-ruby'
require 'rack/response'
require 'uri'

module Bookwatch
  class MailSender
    def initialize(username, api_key, config={})
      @config = config
      @client = SendGrid::Client.new(api_user: username, api_key: api_key, raise_exceptions: false)
    end

    def send_mail(params)
      params = whitelisted_params(params)
      @subject = assemble_subject(params[:page_url])
      @body = assemble_body(**params)
      @mail = SendGrid::Mail.new(merged_options)

      response = client.send(@mail)
      Rack::Response.new(response.body, response.code, response.headers)
    end

    def assemble_subject(page_url)
      "[Feedback] New feedback submitted for #{URI.parse(page_url).host}"
    end

    def assemble_body(helpful: nil, comments: nil, date: nil, page_url: nil)
      <<-EOT
Dear docs writer,

You just received new feedback.

The sender thought the document was #{helpfulness(helpful)}.

Date: #{date}

Page URL: #{page_url}

Comments:
#{comments || "None given"}

Happy editing!
      EOT
    end

    private

    attr_reader :config, :client

    def whitelist
      %w{helpful comments date page_url}
    end

    def whitelisted_params(params)
      {}.tap do |hash|
        params.each do |key, value|
          hash[key.to_sym] = value if whitelist.include?(key)
        end
      end
    end

    def merged_options
      config.merge({text: @body, subject: @subject})
    end

    def helpfulness(was_helpful)
      was_helpful ? 'helpful' : 'not helpful'
    end
  end
end
